/*
     This file is part of GNUnet.
     (C) 2010 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file src/fs/gnunet-fs-gtk.c
 * @brief Main function of gnunet-fs-gtk
 * @author Christian Grothoff
 */
#include "gnunet-fs-gtk-common.h"
#include "gnunet-fs-gtk-event_handler.h"

/**
 * Should gnunet-fs-gtk start in tray mode?
 */
static int tray_only;

/**
 * Handle to our main loop.
 */
static struct GNUNET_GTK_MainLoop *ml;

/**
 * Handle for file-sharing operations.
 */
static struct GNUNET_FS_Handle *fs;

/**
 * List of plugins for meta data extraction.
 */
static struct EXTRACTOR_PluginList *plugins;


/**
 * Return handle for file-sharing operations.
 *
 * @return NULL on error
 */
struct GNUNET_FS_Handle *
GNUNET_FS_GTK_get_fs_handle ()
{
  return fs;
}


/**
 * Get LE plugin list.
 */
struct EXTRACTOR_PluginList *
GNUNET_FS_GTK_get_le_plugins ()
{
  return plugins;
}


/**
 * Get cfg.
 */
const struct GNUNET_CONFIGURATION_Handle *
GNUNET_FS_GTK_get_configuration (void)
{
  return GNUNET_GTK_main_loop_get_configuration (ml);
}


/**
 * Get an object from the main window.
 *
 * @param name name of the object
 * @return NULL on error
 */
GObject *
GNUNET_FS_GTK_get_main_window_object (const char *name)
{
  return GNUNET_GTK_main_loop_get_object (ml, name);
}


/**
 * Task run on shutdown.
 * FIXME: does this need to be a separate task!?
 */
static void
shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
{
  GNUNET_GTK_main_loop_quit (ml);
  if (fs != NULL)
  {
    GNUNET_FS_stop (fs);
    fs = NULL;
  }
  EXTRACTOR_plugin_remove_all (plugins);
  plugins = NULL;
}


/**
 * Callback invoked if the application is supposed to exit.
 */
void
GNUNET_GTK_quit_cb (GObject * object, gpointer user_data)
{
  GNUNET_GTK_tray_icon_destroy ();
  GNUNET_SCHEDULER_add_with_priority (GNUNET_SCHEDULER_PRIORITY_IDLE,
				      &shutdown_task, NULL);
}


/**
 * Search selected in 'file' menu. (from main_window_file_search.c)
 */
void
GNUNET_GTK_main_menu_file_search_activate_cb (GtkWidget * dummy, gpointer data);


/**
 * Add the tab with the 'new' icon for starting a search.
 */
static void
add_new_tab ()
{
  GtkNotebook *notebook;
  GtkWindow *sf;
  gint pages;
  GtkBuilder *builder;
  GtkWidget *label;
  GtkWidget *frame;

  builder =
      GNUNET_GTK_get_new_builder ("gnunet_fs_gtk_main_tab_new_frame.glade");

  /* load frame */
  sf = GTK_WINDOW (gtk_builder_get_object (builder, "_main_tab_new_frame"));
  label = gtk_bin_get_child (GTK_BIN (sf));
  g_object_ref (G_OBJECT (label));
  gtk_container_remove (GTK_CONTAINER (sf), label);
  gtk_widget_destroy (GTK_WIDGET (sf));
  g_object_unref (builder);
  g_signal_connect (G_OBJECT (label), "clicked",
                    G_CALLBACK (&GNUNET_GTK_main_menu_file_search_activate_cb),
                    NULL);

  notebook =
      GTK_NOTEBOOK (GNUNET_FS_GTK_get_main_window_object
                    ("GNUNET_GTK_main_window_notebook"));
  pages = gtk_notebook_get_n_pages (notebook);
  frame = gtk_label_new ("");
  gtk_widget_show (frame);
  gtk_notebook_append_page (notebook, frame, label);
  gtk_notebook_set_current_page (notebook, pages);
  gtk_widget_show (GTK_WIDGET (notebook));
}


/**
 * Actual main function run right after GNUnet's scheduler
 * is initialized.  Initializes up GTK and Glade.
 */
static void
run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
{
  GtkWidget *main_window;

  ml = cls;

  GNUNET_GTK_set_icon_search_path ();
  GNUNET_GTK_setup_nls ();

  /* setup main window */
  main_window =
      GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object
                  ("GNUNET_GTK_main_window"));
  gtk_window_maximize (GTK_WINDOW (main_window));
  GNUNET_GTK_tray_icon_create (GTK_WINDOW (main_window),
                               "gnunet-gtk" /* FIXME: rename icon? */ ,
                               "gnunet-fs-gtk");

  /* initialize file-sharing */
  plugins = EXTRACTOR_plugin_add_defaults (EXTRACTOR_OPTION_DEFAULT_POLICY);
  fs = GNUNET_FS_start (GNUNET_GTK_main_loop_get_configuration (ml),
                        "gnunet-gtk", &GNUNET_GTK_fs_event_handler, NULL,
                        GNUNET_FS_FLAGS_NONE
                        /* fixme later for persistence/probes */ ,
                        /* set other options here later! */
                        GNUNET_FS_OPTIONS_END);
  if (fs != NULL)
  {
    add_new_tab ();
  }
  else
  {
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_create_pseudonym")));
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_advertise_pseudonym")));
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_publish")));
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_search")));
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_download_uri")));
    gtk_widget_hide (GTK_WIDGET
                     (GNUNET_FS_GTK_get_main_window_object
                      ("GNUNET_GTK_main_menu_file_open_gnunet_directory")));
    /* FIXME: set warning in status bar... */
  }
  /* make GUI visible */
  if (!tray_only)
  {
    gtk_widget_show (main_window);
    gtk_window_present (GTK_WINDOW (main_window));
  }

}


int
main (int argc, char *const *argv)
{
  static struct GNUNET_GETOPT_CommandLineOption options[] = {
    {'t', "tray", NULL,
     gettext_noop ("start in tray mode"), 0,
     &GNUNET_GETOPT_set_one, &tray_only},
    GNUNET_GETOPT_OPTION_END
  };

  if (GNUNET_OK !=
      GNUNET_GTK_main_loop_start ("gnunet-fs-gtk", "GTK GUI for GNUnet", argc,
                                  argv, options,
                                  "gnunet_fs_gtk_main_window.glade", &run))
    return 1;
  return 0;
}


/* end of gnunet-fs-gtk.c */
